import logging
import sys
import warnings

from django         import http
from django.conf    import settings
from django.core    import exceptions

from log_handler import DatabaseLogHandler

DEFAULT_IGNORES = (http.Http404, exceptions.PermissionDenied)

class ExceptionLoggingMiddleware(object):
    def __init__(self):
        msg = "ExceptionLoggingMiddleware is deprecated; Remove it and " + \
            "add DatabaseLogHandler to your logging config instead"
        warnings.warn(msg, DeprecationWarning)
        if settings.DEBUG:
            raise exceptions.MiddlewareNotUsed
        self.log_handler = DatabaseLogHandler()

    def process_exception(self, request, _):
        record = logging.makeLogRecord({"exc_info": sys.exc_info(),
                                        "request": request})
        self.log_handler.emit(record)



import logging
import sys
import warnings

from django         import http
from django.core    import exceptions


DEFAULT_IGNORES = (http.Http404, exceptions.PermissionDenied)

class DatabaseLogHandler(logging.Handler):
    def __init__(self):
        super(DatabaseLogHandler, self).__init__()
        self.ignores_processed = False

    def process_ignore_list(self):
        """Defer ignore list evaluation to avoid a circular import of settings
        Only necessary for Django < 1.5"""
        from django.conf import settings
        ignores = getattr(settings, 'SHOOGIE_IGNORE_EXCEPTIONS', None)
        if ignores:
            self.ignores = []
            for exception_path in ignores:
                try:
                    mod, exc = exception_path.rsplit('.', 1)
                    __import__(mod)
                    ignore_mod = sys.modules[mod]
                    ignore_exc = getattr(ignore_mod, exc)
                    self.ignores.append(ignore_exc)
                except (ValueError, ImportError, AttributeError):
                    warnings.warn("Unable to import exception: {}".format(exception_path))
        else:
            self.ignores = DEFAULT_IGNORES
        self.ignores_processed = True

    def emit(self, record):
        if not self.ignores_processed:
            self.process_ignore_list()
        request = getattr(record, 'request', None)
        if request is None:
            request = getattr(record, 'msg', None)
        exc_type, exc_val, tb = getattr(record, 'exc_info', (None, None, None))
        if issubclass(exc_type, self.ignores):
            return
        # Cannot assume db is accessible here
        try:
            # Deferred to avoid circular import of settings (Django < 1.5)
            from . import logger
            logger.log_exception(request, exc_type, exc_val, tb)
        except:
            pass



from django.http  import HttpResponse
from django.views.generic import detail

from shoogie import models

class TechnicalResponseView(detail.BaseDetailView):
    queryset = models.ServerError.objects.all()
    def render_to_response(self, context):
        return HttpResponse(context['servererror'].technical_response)




from django.db      import models
from django.contrib.auth.models import User
"""
Based on http://stackoverflow.com/questions/7130985/#answer-7579467

"""

class ServerError(models.Model):
    timestamp       = models.DateTimeField(auto_now_add=True)

    # http request information
    hostname        = models.CharField(max_length=64)
    request_method  = models.CharField(max_length=10)
    request_path    = models.CharField(max_length=1024)
    query_string    = models.TextField(blank=True)
    post_data       = models.TextField(blank=True)
    cookie_data     = models.TextField(blank=True)
    session_id      = models.CharField(max_length=64)
    session_data    = models.TextField(blank=True)
    user            = models.ForeignKey(User, blank=True, null=True)

    # traceback
    exception_type  = models.CharField(max_length=128)
    exception_str   = models.TextField()
    source_file     = models.CharField(max_length=256)
    source_line_num = models.IntegerField()
    source_function = models.CharField(max_length=128)
    source_text     = models.CharField(max_length=256)

    # django's error page
    technical_response = models.TextField()

    # extra
    issue           = models.CharField(max_length=256, blank=True)
    resolved        = models.BooleanField(default=False)

    def __unicode__(self):
        return '%s %s%s' % (self.timestamp.strftime('%Y-%b-%d %H:%M'),
                                self.hostname, self.request_path)





import re
import socket
import sys
import traceback

from django         import http
from django.conf    import settings
from django.views   import debug

from . import models, utils


def log_exception(request=None, exc_type=None, exc_val=None, tb=None):
    django_request = None
    if isinstance(request, http.HttpRequest):
        django_request = request

    if tb is None:
        exc_type, exc_val, tb = sys.exc_info()

    reporter = debug.ExceptionReporter(django_request,
                                        exc_type,
                                        exc_val,
                                        Traceback(tb) or None)

    tb_desc = traceback.extract_tb(tb)
    tb_file, tb_line_num, tb_function, tb_text = tb_desc[-1]

    data = {}
    if request is not None:
        data = request_data(request)

    data.update({
        'exception_type' : exc_type.__name__,
        'exception_str'  : str(exc_val),
        'source_file'    : utils.TruncateBeginning(tb_file),
        'source_line_num': tb_line_num,
        'source_function': tb_function,
        'source_text'    : tb_text,
    })

    kwargs = utils.clean_data_for_insert(data, models.ServerError)
    models.ServerError.objects.create(
            issue = '',
            technical_response = get_technical_response(reporter),
            **kwargs
        )

def get_technical_response(reporter):
    try:
        return reporter.get_traceback_html()
    except:
        return """<h1>Unable to generate django debug!</h1>
                  <p>Please report the following error on the <a
                  href="https://bitbucket.org/altaurog/django-shoogie/issues"
                  >django-shoogie bugtracker</a>:</p> <pre>%s</pre>
                  """ % traceback.format_exc()

def request_data(request):
    """
    Get data out of the request object.  Assume nothing
    """
    # Allow a string for convenience
    if isinstance(request, basestring):
        return {'request_path': request}

    data = {}
    try:
        data['hostname'] = request.get_host()
    except:
        data['hostname'] = socket.getfqdn()

    data['request_method'] = getattr(request, 'method', '')
    data['request_path'] = getattr(request, 'path', '')

    try:
        data['query_string'] = request.META.get('QUERY_STRING', '')
    except:
        data['query_string'] = ''

    try:
        data['post_data'] = dict(request.POST)
    except:
        data['post_data'] = ''

    try:
        data['cookie_data'] = request.COOKIES
    except:
        data['cookie_data'] = ''

    try:
        data['session_id'] = request.session.session_key
    except:
        data['session_id'] = ''

    try:
        data['session_data'] = dict(request.session.iteritems())
    except:
        data['session_data'] = ''

    try:
        user = request.user
        if user.is_anonymous():
            user = None
    except:
        user = None

    data['user'] = user
    return data

DEFAULT_EXCLUDE = (
        ('/django/core/handlers/base.py$', '^get_response$'),
        ('/django/template/', 'render'),
    )

def compile_exclude(exclude_desc):
    return map(re.compile, exclude_desc)

class Traceback(object):
    "Wrap a traceback object so we can filter it"
    exclude = None
    def __init__(self, tb):
        if self.exclude is None:
            self.load_config()
        if tb:
            tb = self.get_next(tb)
        self.tb = tb
        if tb:
            self.tb_frame = tb.tb_frame
            self.tb_lineno = tb.tb_lineno
            self.tb_lasti = tb.tb_lasti

    @property
    def tb_next(self):
        return self.__class__(self.tb.tb_next) or None

    def __nonzero__(self):
        return bool(self.tb)

    def get_next(self, tb):
        while tb and self.skip(tb):
            tb = tb.tb_next
        return tb

    def skip(self, tb):
        "Determine if this tb should be skipped or not"
        co = tb.tb_frame.f_code
        filename = co.co_filename
        funcname = co.co_name
        for file_re, func_re in self.exclude:
            if file_re.search(filename) and  func_re.search(funcname):
                return True

    def load_config(self):
        "Load exclude configuration into class if necessary"
        exclude_conf = getattr(settings,
                                'SHOOGIE_TRACEBACK_EXCLUDE',
                                DEFAULT_EXCLUDE)
        Traceback.exclude = map(compile_exclude, exclude_conf)



from django.utils   import encoding
from django.db.models import fields

class TruncateBeginning(unicode):
    "Mark string for truncation at beginning"

def truncate(strval, max_length, at_beginning=None):
    strval = encoding.force_unicode(strval)
    if len(strval) <= max_length:
        return strval
    if at_beginning is None and isinstance(strval, TruncateBeginning):
        at_end = False
    else:
        at_end = not(at_beginning)

    if at_end:
        return strval[:max_length - 3] + u'...'
    else:
        return u'...' + strval[-max_length + 3:]

def clean_data_for_insert(data, model):
    """
    Make sure the data will fit in our fields
    """
    clean_data = {}
    for f in model._meta.local_fields:
        try:
            val = data[f.name]
        except KeyError:
            continue

        if isinstance(f, (fields.CharField, fields.TextField)):
            if val is not None:
                if not isinstance(val, basestring):
                    val = repr(val)
                if f.max_length is not None:
                    val = truncate(val, f.max_length)
        clean_data[f.name] = val
    return clean_data




from django.contrib import admin
from django.contrib.admin.views.main import ChangeList
try:
    from django.conf.urls import patterns, url
except ImportError:
    from django.conf.urls.defaults import patterns, url
from django.core.urlresolvers import reverse
from django.http import HttpResponse

from . import models, utils, views

def get_name(user):
    return user.get_full_name() or user.username

class Truncate(object):
    max_length = 50
    def __init__(self, attrname, max_length=None):
        self.attrname = attrname
        self.__name__ = attrname
        if max_length is not None:
            self.max_length = max_length

    def __call__(self, instance):
        val = getattr(instance, self.attrname, '')
        return utils.truncate(val, self.max_length)

class FasterChangeList(ChangeList):
    "Defers large fields we don't use"
    defer_fields = (
                'post_data',
                'cookie_data',
                'session_data',
                'technical_response',
            )
    # get_query_set() was renamed to get_queryset() in Django 1.6
    def get_queryset(self, *args, **kwargs):
        changelist = super(FasterChangeList, self)
        if hasattr(changelist, 'get_queryset'):
            qset = changelist.get_queryset(*args, **kwargs)
        else:
            qset = changelist.get_query_set(*args, **kwargs)
        return qset.defer(*self.defer_fields)

    get_query_set = get_queryset

class ServerErrorAdmin(admin.ModelAdmin):
    list_display = (Truncate('exception_type', 40),
                    Truncate('exception_str', 80),
                    'path_link',
                    'error_date_format',
                    'user_link',
                    'technicalresponse_link',
                    'resolved',)
    date_hierarchy = 'timestamp'
    search_fields  = ('request_path', 'exception_type', 'exception_str', 'source_file', 'source_function', 'source_text')
    actions = ('get_email_list', 'resolve_servererror', 'unresolve_servererror')

    exclude = ('technical_response',)
    readonly_fields = (
            'timestamp',
            'hostname',
            'request_method',
            'request_path',
            'query_string',
            'post_data',
            'cookie_data',
            'session_id',
            'session_data',
            'user',
            'exception_type',
            'exception_str',
            'source_file',
            'source_line_num',
            'source_function',
            'source_text',
        )
    # queryset() was renamed to get_queryset() in Django 1.6
    def get_queryset(self, request):
        model_admin = super(ServerErrorAdmin, self)
        if hasattr(model_admin, 'get_queryset'):
            qset = model_admin.get_queryset(request)
        else:
            qset = model_admin.queryset(request)
        return qset.select_related('user')

    queryset = get_queryset

    def get_changelist(self, request, **kwargs):
        return FasterChangeList

    def error_date_format(self, instance):
        return instance.timestamp.strftime('%Y-%b-%d %H:%M')
    error_date_format.admin_order_field = 'timestamp'
    error_date_format.short_description = 'timestamp'

    get_request_path = Truncate('request_path', 40)
    def path_link(self, instance):
        request_path = self.get_request_path(instance)
        if 'GET' != instance.request_method:
            if instance.request_method:
                return u'%s (%s)' % (request_path, instance.request_method)
            else:
                return request_path
        url = u'http://%s%s?%s' % (instance.hostname, instance.request_path, instance.query_string)
        return u'<a href="{0}" title="{0}">{1}</a>'.format(url, request_path)
    path_link.admin_order_field = 'request_path'
    path_link.allow_tags = True
    path_link.short_description = 'path'

    def user_link(self, instance):
        if not instance.user:
            return u'(None)'
        user = instance.user
        url = reverse('admin:auth_user_change', args=(user.id,))
        templ = u'<a href="{url}" title="{name}">{username}</a>'
        return templ.format(url=url, username=user.username, name=get_name(user))
    user_link.admin_order_field = 'user'
    user_link.allow_tags = True
    user_link.short_description = 'user'

    def get_email_list(self, request, queryset):
        emails = set()
        for se in queryset.select_related('user'):
            user = se.user
            if user and user.email:
                name = get_name(user)
                emails.add('"%s" <%s>' % (name, user.email))
        return HttpResponse(',\n'.join(emails), mimetype='text/plain')
    get_email_list.short_description = 'Get user email addresses for selected errors'

    def technicalresponse_link(self, instance):
        tr_url = reverse('admin:shoogie_technicalresponse', kwargs={'pk':instance.pk})
        return '<a href="%s"><b>debug</b></a>' % tr_url
    technicalresponse_link.allow_tags = True
    technicalresponse_link.short_description = 'Debug'

    def resolve_servererror(self, request, queryset):
        update_count = queryset.update(resolved=True)
        plural = 's' if update_count != 1 else ''
        self.message_user(request, "Marked %d error%s as resolved" % (update_count, plural))
    resolve_servererror.short_description = "Mark selected errors as resolved"

    def unresolve_servererror(self, request, queryset):
        update_count = queryset.update(resolved=False)
        plural = 's' if update_count != 1 else ''
        self.message_user(request, "Marked %d error%s as not resolved" % (update_count, plural))
    unresolve_servererror.short_description = "Mark selected errors as NOT resolved"

    def get_urls(self):
        myview = views.TechnicalResponseView.as_view()
        myurls = patterns('',
            url(r'(?P<pk>\d+)/technicalresponse/$',
                self.admin_site.admin_view(myview, cacheable=True),
                name='shoogie_technicalresponse',
            ),
        )
        return myurls + super(ServerErrorAdmin, self).get_urls()

admin.site.register(models.ServerError, ServerErrorAdmin)



from os.path import join, dirname
from setuptools import setup

package_name = "shoogie"
base_dir = dirname(__file__)

def read(filename):
    f = open(join(base_dir, filename))
    return f.read()

def get_version(package_name, default='0.1'):
    try:
        f = open(join(base_dir, package_name, '__init__.py'))
    except IOError:
        try:
            f = open(join(base_dir, package_name + '.py'))
        except IOError:
            return default
    scope = {}
    exec f in scope
    return scope.get('__version__', default)

setup(
    name = "django-shoogie",
    version = get_version(package_name),
    description = "Log server errors to database",
    long_description = read("README.rst"),
    author = "Aryeh Leib Taurog",
    author_email = "python@aryehleib.com",
    license = 'MIT',
    url = "http://bitbucket.org/altaurog/django-shoogie",
    packages = [package_name],
    install_requires = ["django>=1.3"],
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Environment :: Web Environment",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: System :: Logging",
        "Topic :: Database",
        "Framework :: Django",
    ],
)
